<?php
// $Id: usability_suite.node.inc,v 1.18 2008/06/18 21:46:07 boombatower Exp $
/**
 * @file
 * Provide nodeapi implementation.
 *
 * Copyright 2008 by Jimmy Berry ("boombatower", http://drupal.org/user/214218)
 */

/**
 * Implementation of hook_node_info().
 */
function usability_suite_node_info() {
  return array(
    UTS_STUDY => array(
      'name' => t('UTS Study'),
      'module' => 'usability_suite',
      'description' => t('A study is the basic unit of the Usability Testing Suite.
                          Participants complete a study and the results are evaluated.'),
      'body_label' => t('Description'),
      'custom' => TRUE,
      'locked' => TRUE
    ),
    UTS_TASK => array(
      'name' => t('UTS Task'),
      'module' => 'usability_suite',
      'description' => t('An item to be complete by a participant as part of a study.'),
      'body_label' => t('Description'),
      'custom' => TRUE,
      'locked' => TRUE
    )
  );
}

/**
 * Implementation of hook_form().
 */
function usability_suite_form(&$node) {
  $form = array();
  $type = node_get_types('type', $node);

  if ($type->has_title) {
    $form['title'] = array(
      '#type' => 'textfield',
      '#title' => check_plain($type->title_label),
      '#required' => TRUE,
      '#default_value' => $node->title,
      '#weight' => -10
    );
  }

  if ($type->has_body) {
    $form['body_field'] = node_body_field($node, $type->body_label, $type->min_word_count);
  }

  if ($type->orig_type == UTS_STUDY) {
    $form['study_type'] = array(
      '#type' => 'select',
      '#title' => t('Type'),
      '#description' => t('The type of study that is to be offered.'),
      '#options' => array(
        UTS_STUDY_TYPE_TASK
      ),
      '#weight' => -9,
      '#default_value' => (isset($node->study_type) ? $node->study_type : '')
    );
    $form['study_type']['#options'] = drupal_map_assoc($form['study_type']['#options'], 'usability_suite_get_study_type');

    $form['participant_count'] = array(
      '#type' => 'textfield',
      '#title' => t('Number of participants'),
      '#description' => t('The maximum number of participants that can participate in the study.'),
      '#size' => 4,
      '#maxlength' => 4,
      '#required' => TRUE,
      '#weight' => -8,
      '#default_value' => (isset($node->participant_count) ? $node->participant_count : '')
    );
    $form['body_field']['#weight'] = -7;
    $form['audience'] = array(
      '#type' => 'fieldset',
      '#title' => t('Audience'),
      '#description' => t('Define what type of participants are relevant for the study.'),
      '#weight' => -6
    );
    $form['audience']['audience_role'] = array(
      '#type' => 'select',
      '#title' => t('Role'),
      '#options' => array(
        UTS_STUDY_AUDIENCE_ROLE_USER,
        UTS_STUDY_AUDIENCE_ROLE_DESIGNER,
        UTS_STUDY_AUDIENCE_ROLE_PROGRAMER
      ),
      '#weight' => -10,
      '#default_value' => (isset($node->audience_role) ? $node->audience_role : '')
    );
    $form['audience']['audience_role']['#options'] = drupal_map_assoc($form['audience']['audience_role']['#options'], 'usability_suite_get_study_audience_role');

    $form['audience']['audience_level'] = array(
      '#type' => 'select',
      '#title' => t('Experience level'),
      '#options' => array(
        UTS_STUDY_AUDIENCE_LEVEL_BEGINNER,
        UTS_STUDY_AUDIENCE_LEVEL_INTERMEDIATE,
        UTS_STUDY_AUDIENCE_LEVEL_ADVANCED
      ),
      '#weight' => -9,
      '#default_value' => (isset($node->audience_level) ? $node->audience_level : '')
    );
    $form['audience']['audience_level']['#options'] = drupal_map_assoc($form['audience']['audience_level']['#options'], 'usability_suite_get_study_audience_level');
  }
  else if ($type->orig_type == UTS_TASK) {
    if (is_numeric($study_nid = arg(3))) {
      // Task will be added to a study.
      $form['study_nid'] = array(
        '#type' => 'value',
        '#value' => $study_nid
      );
      usability_suite_set_title($study_nid);
    }

    $form['body_field']['#weight'] = -9;
    $form['max_time'] = array(
      '#type' => 'textfield',
      '#title' => t('Maximum time'),
      '#description' => t('The maximum time a participant will be allowed for the task in seconds.'),
      '#size' => 6,
      '#maxlength' => 6,
      '#required' => TRUE,
      '#weight' => -8,
      '#default_value' => (isset($node->max_time) ? $node->max_time : '')
    );
    $form['steps'] = array(
      '#type' => 'fieldset',
      '#title' => t('Steps'),
      '#description' => t('The ideal path to complete the task.'),
      '#weight' => -7
    );
    $form['steps']['list'] = array(
      '#type' => 'item',
      '#value' => usability_suite_render_steps(),
      '#weight' => -10
    );
    $form['steps']['open_ideal_recording'] = array(
      '#type' => 'submit',
      '#value' => t('Open recorder'),
      '#submit' => array('usability_suite_open_recorder'),
      '#weight' => -9,
      '#disabled' => isset($_SESSION['uts_recording_ideal_steps'])
    );
  }
  $form['#submit'] = array('usability_suite_redirect');

  return $form;
}

/**
 * Set the appropriate redirect when a UTS node form is submitted.
 */
function usability_suite_redirect($form, &$form_state) {
  if (isset($form_state['values']['nid'])) {
    // On update redirect to study overview page.
    $form_state['redirect'] = 'admin/uts/studies';
  }
  else {
    // On create redirect to add tasks.
    $form_state['redirect'] = "admin/uts/tasks/{$form_state['values']['nid']}";
  }
  drupal_set_message($form_state['redirect']);
}

function usability_suite_render_steps() {
  $out = '<i>None</i>';
  if (isset($_SESSION['uts_ideal_steps']) && is_array($_SESSION['uts_ideal_steps'])) {
    $list = array();
    foreach ($_SESSION['uts_ideal_steps'] as $step) {
      $list[] = $step[0] . (!empty($step[1]) ? ' - ' . $step[1] : '');
    }
    $out = theme('item_list', $list);
  }
  return $out;
}

/**
 * Open the ideal step recorder.
 */
function usability_suite_open_recorder($form, &$form_state) {
  $_SESSION['uts_recording_ideal_steps'] = FALSE;
  $_SESSION['uts_ideal_steps'] = array();
}

/**
 * Output the ideal step recorder if running.
 *
 * @return HTML output.
 */
function usability_suite_footer() {
  if (isset($_SESSION['uts_recording_ideal_steps'])) {
    $op = isset($_POST['op']) ? $_POST['op'] : '';

    // Record steps if not performing a record action and currently set to record.
    if ($_SESSION['uts_recording_ideal_steps'] &&
        $op != t('Start') &&
        $op != t('Stop') &&
        $op != t('Clear') &&
        $op != t('Close')) {
      $_SESSION['uts_ideal_steps'][] = array($_GET['q'], strip_tags(theme('breadcrumb', drupal_get_breadcrumb())));
    }
  return '
<div id="uts_record_overlay">
' . drupal_get_form('usability_suite_record_actions') . '
</div>
';
  }
}

/**
 * Ideal step recorder actions form.
 */
function usability_suite_record_actions() {
  $recording = $_SESSION['uts_recording_ideal_steps'];
  $form = array();
  $form['start_recording'] = array(
    '#type' => 'submit',
    '#value' => t('Start'),
    '#weight' => -10,
    '#disabled' => $recording
  );
  $form['stop_recording'] = array(
    '#type' => 'submit',
    '#value' => t('Stop'),
    '#weight' => -9,
    '#disabled' => !$recording
  );
  $form['clear_recording'] = array(
    '#type' => 'submit',
    '#value' => t('Clear'),
    '#weight' => -8,
    '#disabled' => !(count($_SESSION['uts_ideal_steps']) > 0)
  );
  $form['close_recording'] = array(
    '#type' => 'submit',
    '#value' => t('Close'),
    '#weight' => -7,
  );
  $form['#submit'] = array('usability_suite_record_actions_submit');
  return $form;
}

/**
 * Ideal step recorder actions hook_submit().
 */
function usability_suite_record_actions_submit($form, &$form_state) {
  if ($form_state['values']['op'] == t('Start')) {
    $_SESSION['uts_recording_ideal_steps'] = TRUE;
  }
  else if ($form_state['values']['op'] == t('Stop')) {
    $_SESSION['uts_recording_ideal_steps'] = FALSE;
  }
  else if ($form_state['values']['op'] == t('Clear')) {
    $_SESSION['uts_ideal_steps'] = array();
  }
  else if ($form_state['values']['op'] == t('Close')) {
    unset($_SESSION['uts_recording_ideal_steps']);
  }
}

/**
 * Implementation of hook_validate().
 */
function usability_suite_validate($node, &$form) {
  $type = node_get_types('type', $node);

  if ($type->orig_type == UTS_STUDY) {
    if (!(is_numeric($node->participant_count) && (int) $node->participant_count == (float) $node->participant_count)) {
      form_set_error('participant_count', t('Participant count must be an integer.'));
    }
  }
  else if ($type->orig_type == UTS_TASK) {
    if (!(is_numeric($node->max_time) && (int) $node->max_time == (float) $node->max_time)) {
      form_set_error('max_time', t('Maximum time must be an integer.'));
    }
  }
}

/**
 * Implementation of hook_load().
 */
function usability_suite_load($node) {
  $type = node_get_types('type', $node);

  if ($type->orig_type == UTS_STUDY) {
    // Load study data.
    $result = db_query('SELECT study_type, participant_count, audience_role, audience_level
                        FROM {usability_suite_study}
                        WHERE vid = %d',
                        $node->vid);
    return db_fetch_object($result);
  }
  else if ($type->orig_type == UTS_TASK) {
    // Load task data.
    $result = db_query('SELECT study_nid, max_time, weight
                        FROM {usability_suite_task}
                        WHERE vid = %d',
                        $node->vid);
    return db_fetch_object($result);
  }
}

/**
 * Implementation of hook_insert().
 */
function usability_suite_insert($node) {
  $type = node_get_types('type', $node);

  if ($type->orig_type == UTS_STUDY) {
    db_query('INSERT INTO {usability_suite_study} (nid, vid, study_type, participant_count, audience_role, audience_level)
              VALUES (%d, %d, %d, %d, %d, %d)',
              $node->nid, $node->vid, $node->study_type, $node->participant_count, $node->audience_role, $node->audience_level);
  }
  else if ($type->orig_type == UTS_TASK) {
    db_query('INSERT INTO {usability_suite_task} (nid, vid, max_time)
              VALUES (%d, %d, %d)',
              $node->nid, $node->vid, $node->max_time);
    if (isset($node->study_nid)) {
      // Insert task into specific study.
      usability_suite_tasks_import($node->study_nid, array($node->nid));
    }
  }
}

/**
 * Implementation of hook_update().
 */
function usability_suite_update($node) {
  if ($node->revision) {
    usability_suite_insert($node);
    return;
  }

  $type = node_get_types('type', $node);

  if ($type->orig_type == UTS_STUDY) {
    db_query('UPDATE {usability_suite_study}
              SET study_type = %d,
                  participant_count = %d,
                  audience_role = %d,
                  audience_level = %d
              WHERE vid = %d',
              $node->study_type, $node->participant_count, $node->audience_role, $node->audience_level, $node->vid);
  }
  else if ($type->orig_type == UTS_TASK) {
    db_query('UPDATE {usability_suite_task}
              SET max_time = %d
              WHERE vid = %d',
              $node->max_time, $node->vid);
  }
}

/**
 * Implementation of hook_delete().
 */
function usability_suite_delete($node) {
  $type = node_get_types('type', $node);

  if ($type->orig_type == UTS_STUDY) {
    db_query('DELETE FROM {usability_suite_study} WHERE nid = %d', $node->nid);
  }
  else if ($type->orig_type == UTS_TASK) {
    db_query('DELETE FROM {usability_suite_task} WHERE nid = %d', $node->nid);
  }
}

/**
 * Implementation of hook_view().
 */
function usability_suite_view($node, $teaser = FALSE, $page = FALSE) {
  $node = node_prepare($node, $teaser);
  $type = node_get_types('type', $node);

  if ($type->orig_type == UTS_STUDY) {
    // Basic meta data.
    $node->content['study_type'] = array(
      '#value' => t('<b>Study type:</b> @type<br />', array('@type' => usability_suite_get_study_type($node->study_type))),
      '#weight' => -10
    );
    $node->content['audience'] = array(
      '#value' => t('<b>Audience:</b> @role (@level)<br />',
                    array('@role' => usability_suite_get_study_audience_role($node->audience_role),
                           '@level' => usability_suite_get_study_audience_level($node->audience_level))),
      '#weight' => -9
    );
    $node->content['participant_count'] = array(
      '#value' => t('<b>Number of participants:</b> @count<br />', array('@count' => $node->participant_count)),
      '#weight' => -8
    );

    if (!$teaser) {
      // Related tasks.
      $tasks = usability_suite_tasks_load($node->nid);
      $node->content['tasks'] = array(
        '#value' => '<b>Tasks:</b><br /><ol>',
        '#weight' => 1
      );
      foreach ($tasks as $task) {
        $node->content['tasks'][] = array(
          '#value' => t('<li><b>!title</b><br />@body</li>', array('!title' => l($task->title, 'node/' . $task->nid), '@body' => $task->body)),
          '#weight' => $task->weight
        );
      }
      $node->content['tasks_close'] = array(
        '#value' => '</ol>',
        '#weight' => 2
      );
    }
  }
  else if ($type->orig_type == UTS_TASK) {
    // Basic meta data.
    $node->content['max_time'] = array(
      '#value' => t('<b>Maximum time:</b> @time<br />', array('@time' => format_interval($node->max_time))),
      '#weight' => -10
    );
  }
  return $node;
}
