<?php
// $Id: usability_suite.pages.inc,v 1.17 2008/06/18 21:11:03 boombatower Exp $
/**
 * @file
 * Provide interface pages.
 *
 * Copyright 2008 by Jimmy Berry ("boombatower", http://drupal.org/user/214218)
 */

/**
 * Task import page.
 *
 * @param interger $study_nid Study NID to import tasks for.
 * @return array Form output.
 */
function usability_suite_import_tasks(&$form_state, $study_nid = NULL) {
  $form = array();
  $form['study_nid'] = array(
    '#type' => 'value',
    '#value' => $study_nid
  );

  if ($study_nid) {
    usability_suite_set_title($study_nid, 'Import tasks');

    // Only import sources from existing database if they will be related to a study.
    $form['existing_source'] = array(
      '#type' => 'fieldset',
      '#title' => t('Existing Source'),
      '#description' => t('Import tasks from database.'),
      '#weight' => -10
    );
    $form['existing_source']['tasks'] = array(
      '#type' => 'select',
      '#title' => t('Tasks'),
      '#multiple' => TRUE,
      '#weight' => -10
    );
    // Add options.
    $tasks = usability_suite_tasks_load(NULL, TRUE);
    $options = array();
    foreach ($tasks as $task) {
      $options[$task->nid] = $task->title;
    }
    $form['existing_source']['tasks']['#options'] = $options;
    $form['existing_source']['tasks']['#size'] = min(6, count($options));
  }

  $form['xml_source'] = array(
    '#type' => 'fieldset',
    '#title' => t('XML Source'),
    '#description' => t('Import tasks from an XML source.'),
    '#weight' => -9
  );
  $form['xml_source']['file'] = array(
    '#type' => 'file',
    '#title' => t('Source'),
    '#description' => t('Correctly formed XML document containing UTS tasks.'),
    '#disabled' => TRUE, // TODO
    '#weight' => -10
  );

  $form['import'] = array(
    '#type' => 'submit',
    '#value' => 'Import',
    '#weight' => -8
  );

  return $form;
}

/**
 * Execute import either from XML or non-related tasks in database.
 */
function usability_suite_import_tasks_submit($form, &$form_state) {
  // Tasks from database.
  if (isset($form_state['values']['tasks']) && count($form_state['values']['tasks']) > 0) {
    usability_suite_tasks_import($form_state['values']['study_nid'], $form_state['values']['tasks']);
  }

  // TODO Tasks from XML.
}

/**
 * Menu callback -- ask for confirmation of task freeing then redirect
 * to appropriate overview page.
 */
function usability_suite_task_free_confirm(&$form_state, $task) {
  $type = node_get_types('type', $task);
  if ($type->orig_type != UTS_TASK) {
    drupal_set_message(t('Must be task to free from study.'));
    drupal_goto('admin/uts/tasks');
  }
  $study = node_load($task->study_nid);
  $redirect = 'admin/uts/tasks/' . $study->nid;

  // Inject form values.
  $form['task_nid'] = array(
    '#type' => 'value',
    '#value' => $task->nid,
  );
  $form['redirect'] = array(
    '#type' => 'value',
    '#value' => $redirect,
  );

  return confirm_form($form,
    t('Are you sure you want to free %task from %study?', array('%task' => $task->title, '%study' => $study->title)),
    isset($_GET['destination']) ? $_GET['destination'] : $redirect,
    t('This action cannot be undone.'),
    t('Free'),
    t('Cancel')
  );
}

/**
 * Execute task freeing.
 */
function usability_suite_task_free_confirm_submit($form, &$form_state) {
  if ($form_state['values']['confirm']) {
    $task = node_load($form_state['values']['task_nid']);
    $study = node_load($task->study_nid);
    db_query('UPDATE {usability_suite_task}
                SET study_nid = NULL,
                weight = NULL
                WHERE nid = %d', $task->nid);
    drupal_set_message(t('%task was freed from %study.', array('%task' => $task->title, '%study' => $study->title)));
  }

  // Set the redirect to the pre-determined location.
  $form_state['redirect'] = $form_state['values']['redirect'];
}

/**
 * Menu callback -- ask for confirmation of node deletion then redirect
 * to appropriate overview page.
 */
function usability_suite_delete_confirm(&$form_state, $node) {
  // Decide on the redirect url.
  $type = node_get_types('type', $node);
  if ($type->orig_type == UTS_STUDY) {
    $redirect = 'admin/uts/studies';
  }
  else if ($type->orig_type == UTS_TASK) {
    // TODO If task is related to study then return to tasks overview for that study.
    $redirect = 'admin/uts/tasks';
  }

  // Inject form values.
  $form['nid'] = array(
    '#type' => 'value',
    '#value' => $node->nid,
  );
  $form['redirect'] = array(
    '#type' => 'value',
    '#value' => $redirect,
  );

  return confirm_form($form,
    t('Are you sure you want to delete %title?', array('%title' => $node->title)),
    isset($_GET['destination']) ? $_GET['destination'] : $redirect,
    t('This action cannot be undone.'),
    t('Delete'),
    t('Cancel')
  );
}

/**
 * Execute node deletion.
 */
function usability_suite_delete_confirm_submit($form, &$form_state) {
  if ($form_state['values']['confirm']) {
    node_delete($form_state['values']['nid']);
  }

  // Set the redirect to the pre-determined location.
  $form_state['redirect'] = $form_state['values']['redirect'];
}
